<?php
defined('ABSPATH') or exit;

if (!class_exists('MontonioFinancingSDK')) {
    require_once dirname(dirname(__FILE__)) . '/libraries/MontonioFinancingSDK.php';
}

if (!class_exists('MontonioOrderPrefixer')) {
    require_once dirname(dirname(__FILE__)) . '/libraries/MontonioOrderPrefixer.php';
}

if (!class_exists('MontonioHelper')) {
    require_once dirname(dirname(__FILE__)) . '/libraries/MontonioHelper.php';
}

if (!class_exists('Montonio_Financing_Settings')) {
    require_once 'class-montonio-financing-settings.php';
}

class Montonio_Financing extends WC_Payment_Gateway
{
    /**
     * Montonio image location
     */
    const MONTONIO_LOGO_URI = 'https://public.montonio.com/logo/financing/ee-financing.png';

    /**
     * Local language delimiter
     */
    const LANG_DELIMITER = '_';

    /**
     * Prevent typos
     */
    const NONCE_HIREPURCHASE_CALLBACK = 'montonio-financing-callback-';

    /**
     * WC_Gateway_Montonio constructor.
     */
    public function __construct()
    {
        $this->id                 = 'montonio';
        $this->has_fields         = true;
        $this->method_title       = __('Montonio Financing', 'montonio-for-woocommerce');
        $this->method_description = __('Allows customers to apply for financing through Montonio.', 'montonio-for-woocommerce');
        $this->icon = 'https://public.montonio.com/logo/financing/en-financing.png';
        $this->init_form_fields();
        $this->init_settings();

        $this->logo_locale = WC_Montonio_Helper::get_locale(apply_filters('wpml_current_language', get_locale()));

        if ($this->logo_locale == 'et') {
            $this->icon = 'https://public.montonio.com/logo/financing/ee-financing.png';
        } elseif ($this->logo_locale == 'lt') {
            $this->icon = 'https://public.montonio.com/logo/financing/lt-financing.png';
        }
    
        $this->title = __($this->get_option( 'titlev2', __('Compare financing offers', 'montonio-for-woocommerce') ), 'montonio-for-woocommerce');
        $this->description = __($this->get_option( 'descriptionv2', __('Select the financing offer you like best! With one application you receive multiple financing offers.', 'montonio-for-woocommerce') ), 'montonio-for-woocommerce');


        /**
         * Register the callback function for when returning from Montonio Hirepurchase
         */
        add_action('woocommerce_api_montonio_financing_return', array($this, 'check_financing_response'));

        /**
         * Register payment notification webhook handler
         */
        add_action('woocommerce_api_montonio_financing_notification', array($this, 'check_financing_notification'));

        /**
         * Admin options
         */
        add_action('woocommerce_update_options_payment_gateways_montonio', array($this, 'process_admin_options'));

        /**
         * Check that currency is supported
         */
        add_filter('woocommerce_available_payment_gateways', array($this, 'validate_currency_supported'), 10, 3);

        /**
         * Check the configured minimum amount
         */
        add_filter('woocommerce_available_payment_gateways', array($this, 'validate_minimum_amount'), 10, 3);

        /**
         * Restrict financing checkout for unsupported locales
         */
        add_filter('woocommerce_available_payment_gateways', array($this, 'restrict_unsupported_locales'), 10, 3);
    }

    /**
     * Initialize the inputs the merchant can configure at the Settings screen
     *
     * @return void
     */
    public function init_form_fields()
    {
        $this->setPluginSettings(Montonio_Financing_Settings::create());
        $this->form_fields = $this->getPluginSettings()->getFormFields();
    }

    /**
     * Generate and display the HTML for the form fields
     *
     * @return void
     */
    public function admin_options()
    {
        $this->getPluginSettings()->setLang($this->getLocalLang($this::LANG_DELIMITER));
        $this->getPluginSettings()->setCurrency(get_woocommerce_currency());
        $this->getPluginSettings()->setPublicKey($this->get_option('publicKey'));
        $all_fields = $this->get_form_fields();

        // get form fields for Loan Calculator
        $calculatorOptionsCount = count($this->getPluginSettings()->getCalculatorFormFields());

        $tabs = $this->generateTabs(array(
            array(
                'name'  => __('Montonio Financing Settings', 'montonio-for-woocommerce'),
                'slice' => array_slice($all_fields, 0, count($all_fields) - $calculatorOptionsCount),
            ),
            array(
                'name'  => __('Calculator Widget Settings', 'montonio-for-woocommerce'),
                'slice' => array_slice($all_fields, -$calculatorOptionsCount, count($all_fields)),
            ),
        ));

        $this->getPluginSettings()->buildAdminFormHtml($tabs);
    }

    // =========================================================================
    // Checkout page
    // =========================================================================

    public function validate_currency_supported($available_gateways) {
        if ($this->enabled !== 'yes') {
            return $available_gateways;
        }

        if (!MontonioHelper::isClientCurrencySupported(array('EUR'))) {
            unset($available_gateways['montonio']);
        }

        return $available_gateways;
    }

    public function validate_minimum_amount($available_gateways) {
        if (WC()->cart && WC()->cart->get_total('raw')) {
            $cartTotal = (float) (WC()->cart->get_total('raw'));
            $minCartTotal = $this->get_option('minOrderTotal', 60);
            $minCartTotal = is_numeric($minCartTotal) ? (float)$minCartTotal : 60;
            if ($cartTotal < $minCartTotal) {
                unset($available_gateways['montonio']);
            }
        }

        return $available_gateways;
    }

    public function restrict_unsupported_locales($available_gateways) {
        $lang = apply_filters('wpml_current_language', get_locale());
        $locale = MontonioHelper::getLocale($lang);

        if (in_array($locale, array('fi', 'fi_FI', 'fi_fi', 'fi-FI', 'fi-fi'))) {
            unset($available_gateways['montonio']);
        }

        return $available_gateways;
    }

    /**
     * Resolve Payment handle style
     *
     * @return void
     */
    public function payment_fields()
    {

        wp_enqueue_style(
            'custom-frontend-style',
            WC_MONTONIO_PLUGIN_URL . '/financing/assets/css/payment-handle/default.css'
        );

        echo wpautop(wptexturize($this->description));
    }

    /**
     * The method that executes after pressing "Place Order"
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);
        $order->add_order_note(__('Montonio Financing: Order checkout process is started', 'montonio-for-woocommerce'));

        // TODO: Merchant name support when it becomes available
        $storePrefix = $this->get_option('montonioFinancingOrderPrefix');

        /**
         * Create new Montonio Financing SDK instance
         */
        $montonioFinancing = new MontonioFinancingSDK(
            $this->get_option('publicKey'),
            $this->get_option('secretKey'),
            $this->get_option('sandbox') === 'yes' ? 'sandbox' : 'production'
        );

        /**
         * Prepare Payment Data for Montonio Financing
         */
        $paymentData = array(
            'origin'                    => 'online',
            'currency'                  => $order->get_currency(),
            'merchant_reference'        => (string) MontonioOrderPrefixer::addPrefix($storePrefix, $order_id),
            'checkout_first_name'       => (string) $order->get_billing_first_name(),
            'checkout_last_name'        => (string) $order->get_billing_last_name(),
            'checkout_email'            => (string) $order->get_billing_email(),
            'checkout_phone_number'     => (string) $order->get_billing_phone(),
            'checkout_city'             => (string) $order->get_billing_city(),
            'checkout_address'          => (string) $order->get_billing_address_1(),
            'checkout_postal_code'      => (string) $order->get_billing_postcode(),
            'merchant_notification_url' => add_query_arg('wc-api', 'montonio_financing_notification', home_url('/')),
            'merchant_return_url'       => add_query_arg('wc-api', 'montonio_financing_return', home_url('/')),
            'checkout_products'         => array(),
        );

        /**
         * Add products & shipping to Payment Data
         * Products will have price set to 0 and Cart total will hold the total price to prevent rounding errors
         */
        foreach ($order->get_items() as $item_id => $item_data) {
            $paymentData["checkout_products"][] = array(
                "product_name"  => $item_data->get_product()->get_name(),
                "product_price" => 0,
                "quantity"      => (int) $item_data->get_quantity(),
            );
        }

        $shipping_cost = $order->get_shipping_total();
        if ($shipping_cost && $shipping_cost > 0) {
            $paymentData["checkout_products"][] = array(
                "product_name"  => 'SHIPPING',
                "product_price" => 0,
                "quantity"      => 1,
            );
        }

        $paymentData["checkout_products"][] = array(
            "product_name"  => 'Cart total',
            "product_price" => $order->get_total(),
            "quantity"      => 1,
        );

        /**
         * Add preselected locale if possible
         */
        $wpmlCustomerLanguage = apply_filters('wpml_current_language', null);
        if ($wpmlCustomerLanguage) {
            $paymentData['preselected_locale'] = MontonioHelper::getLocale($wpmlCustomerLanguage);
        } else {
            $paymentData['preselected_locale'] = MontonioHelper::getLocale(get_locale());
        }

        $paymentData = apply_filters('montonio_financing_before_payment_data_submission', $paymentData, $order);

        $montonioFinancing->setPaymentData($paymentData);
        $paymentUrl = $montonioFinancing->getPaymentUrl();

        /**
         * Return response after which redirect to Montonio Financing will happen
         */
        return array(
            'result'   => 'success',
            'redirect' => $paymentUrl,
        );
    }

    /**
     * Handle notification coming from Montonio Financing
     * @since 1.2.0
     *
     * @return void
     */
    public function check_financing_notification()
    {
        return $this->check_financing_response(true);
    }

    /**
     * Check callback from Montonio
     * and redirect user: thankyou page for success, checkout on declined/failure
     *
     * @param bool $is_notification True if webhook notification, false otherwise
     * @since 1.2.0
     *
     * @return void
     */
    public function check_financing_response($is_notification = false)
    {
        global $woocommerce;

        /**
         * Get Payment Token from request
         */
        if (!isset($_REQUEST['payment_token'])) {
            wc_add_notice(__('Unable to finish the payment. Please try again or choose a different payment method.', 'montonio-for-woocommerce'), 'notice');
            wp_redirect(wc_get_checkout_url());
            exit;
        }

        /**
         * Return URL if payment for order failed
         */
        $returnUrl = wc_get_checkout_url();

        /**
         * Decode and Validate Payment Token with Montonio Financing SDK
         */
        $token = sanitize_text_field($_REQUEST['payment_token']);
        try {
            $response = MontonioFinancingSDK::decodePaymentToken($token, $this->get_option('secretKey'));
        } catch (Throwable $exception) {
            wc_add_notice('There was a problem with processing the order. ', 'error');
            if ($is_notification) {
                http_response_code(401);
            } else {
                wp_redirect($returnUrl);
            }
            exit;
        }

        $paymentStatus = sanitize_text_field($response->status);

        /**
         * Get order
         */
        $orderId = MontonioOrderPrefixer::removePrefix(sanitize_text_field($response->merchant_reference));
        $order   = wc_get_order($orderId);

        /**
         * Check that Montonio was used to place the order
         */
        if (!$order || $order->get_payment_method() !== $this->id) {
            http_response_code(404);
            die();
        }

        /**
         * Check if payment successful
         */
        if (!empty( $order->get_date_paid())) {
            $returnUrl = WC_Payment_Gateway::get_return_url($order);
        } else {
            if ($paymentStatus === 'finalized') {
                $order->payment_complete();
                $returnUrl = WC_Payment_Gateway::get_return_url($order);
                $woocommerce->cart->empty_cart();
            } else {
                wc_add_notice(__('Unable to finish the payment. Please try again or choose a different payment method.', 'montonio-for-woocommerce'), 'notice');
            }
        }

        if ($is_notification) {
            http_response_code(200);
        } else {
            wp_redirect($returnUrl);
        }

        exit;
    }

    protected function getLocalLang($delimiter)
    {
        $lang = explode($delimiter, get_locale());

        return $lang[0];
    }

    protected function generateTabs($tabs)
    {
        $data = [];
        foreach ($tabs as $key => $value) {
            $data[$key]['name']  = $value['name'];
            $data[$key]['slice'] = $this->generate_settings_html($value['slice'], false);
        }

        return $data;
    }

    public function validate_secretKey_field($key, $value)
    {
        if (1 > strlen($value)) {
            WC_Admin_Settings::add_error(esc_html__(
                'Secret key must not be empty',
                'montonio-for-woocommerce'
            ));
        }

        return $value;
    }

    /**
     * @return object
     */
    public function getPluginSettings()
    {
        return $this->pluginSettings;
    }

    /**
     * @param object $pluginSettings
     */
    public function setPluginSettings($pluginSettings)
    {
        $this->pluginSettings = $pluginSettings;
    }
}
