<?php
defined('ABSPATH') or exit;

abstract class Montonio_Shipping_Method extends WC_Shipping_Method {
    /**
     * @var Montonio_Shipping_Method_Settings
     */
    protected $pluginSettings;

    /**
     * @var string
     */
    public $id;

    /**
     * @var string
     */
    public $title;

    /**
     * @var string
     */
    public $logo;

    /**
     * @var string
     */
    protected $providerName;

    /**
     * @var string
     */
    protected $type;

    /**
     * @var string
     */
    protected $shippingMethodIdentifier;

    /**
     * @return void
     */
    public function __construct($instanceId = 0)
    {
        parent::__construct($instanceId);
        $this->init($this->instance_id);

        // Save settings in admin
        add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
    }

    /**
     * @return void
     */
    public function init_settings() {
        parent::init_settings();

        $settingsClass = $this->getPluginSettings();
        $settingsClass->setSettings($this->settings);
    }

    /**
     * Called when doing shipping rate calculations
     * @param $package
     * @return void
     */
    public function calculate_shipping($package = []) {
        if (get_option('montonio_shipping_enabled', 'no') !== 'yes') {
            return;
        }

        $cost = 0;
        $cartTotal = isset($package['contents_cost']) ? $package['contents_cost'] : $this->getCartTotal($package);
        if (
            !$this->get_option('enableFreeShippingThreshold') ||
            $this->get_option('enableFreeShippingThreshold') === 'no' ||
            (float)$cartTotal < (float)$this->get_option('freeShippingThreshold')
        ) {
            $cost = $this->get_option('price') ?: 0;
        }

        // Check for free shipping coupon
        $hasFreeShipping = false;
        $appliedCouponCodes = $package['applied_coupons'];
        foreach ($appliedCouponCodes as $couponCode) {
            $coupon = new WC_Coupon($couponCode);
            if ($coupon->get_free_shipping()) {
                $hasFreeShipping = true;
                break;
            }
        }

        $rate = array(
            'label'     => $this->title,
            'cost'      => $hasFreeShipping ? 0 : $cost,
            'calc_tax'  => 'per_order',
            'meta_data' => array(
                'shipping_method_identifier' => $this->shippingMethodIdentifier,
                'provider_name'              => $this->providerName,
                'type'                       => $this->type,
                'method_class_name'          => get_class($this),
                'instance_id'                => $this->instance_id
            )
        );

        $this->add_rate($rate);
    }

    /**
     * @param $package
     * @return bool
     */
    public function isShippingMethodAvailable($package) {
        if ($this->get_option('enablePackageMeasurementsCheck', 'no') === 'no') {
            return true;
        }

        if (
            $this->get_option('hideWhenNoMeasurements', 'no') === 'yes' &&
            $this->doesPackageHaveMissingMeasurements($package)
        ) {
            return false;
        }

        return $this->validatePackageWeight($package) && $this->validatePackageDimensions($package);
    }

    protected function validatePackageWeight($package) {
        return true;
    }

    protected function validatePackageDimensions($dimensions) {
        return true;
    }

    protected function calculatePackageWeight($package) {
        // Used to convert to kg
        $weightMultiplier = Montonio_Shipping::get_weight_multiplier();
        $packageWeight = 0;

        foreach ($package['contents'] as $item) {
            $itemWeight = (float)$item['data']->get_weight() * (int)$item['quantity'] * $weightMultiplier;
            $packageWeight += $itemWeight;
        }

        return $packageWeight;
    }

    protected function calculatePackageDimensions($package) {
        // Used to convert to cm
        $dimensionsMultiplier = Montonio_Shipping::get_dimensions_multiplier();
        $packageDimensions = [0, 0, 0];

        foreach ($package['contents'] as $item) {
            $itemDimensions = [0, 0, 0];
            $itemDimensions[0] = (float)$item['data']->get_length() * $dimensionsMultiplier;
            $itemDimensions[1] = (float)$item['data']->get_width() * $dimensionsMultiplier;
            $itemDimensions[2] = (float)$item['data']->get_height() * $dimensionsMultiplier;
            // Sort from smallest to largest dimension
            sort($itemDimensions);

            if ($itemDimensions[0] && $itemDimensions[0] > $packageDimensions[0]) {
                $packageDimensions[0] = $itemDimensions[0];
            }
            if ($itemDimensions[1] && $itemDimensions[1] > $packageDimensions[1]) {
                $packageDimensions[1] = $itemDimensions[1];
            }
            if ($itemDimensions[2] && $itemDimensions[2] > $packageDimensions[2]) {
                $packageDimensions[2] = $itemDimensions[2];
            }
        }

        return $packageDimensions;
    }

    protected function doesPackageHaveMissingMeasurements($package) {
        // Check if known measurement units are used
        $dimensionsMultiplier = Montonio_Shipping::get_dimensions_multiplier();
        $weightMultiplier = Montonio_Shipping::get_weight_multiplier();

        if (!$dimensionsMultiplier || !$weightMultiplier) {
            return true;
        }

        foreach ($package['contents'] as $item) {
            if (
                !(float)$item['data']->get_length() ||
                !(float)$item['data']->get_width() ||
                !(float)$item['data']->get_height() ||
                !(float)$item['data']->get_weight()
            ) {
                return true;
            }
        }

        return false;
    }

    protected function getCartTotal($package) {
        $total = 0;
        foreach ($package['contents'] as $line) {
            $total += (float)$line['line_total'];
        }

        return $total;
    }

    // =========================================================================
    // Getters and Setters
    // =========================================================================

    /**
     * @return Montonio_Shipping_Method_Settings
     */
    public function getPluginSettings() {
        return $this->pluginSettings;
    }

    /**
     * @param Montonio_Shipping_Method_Settings $pluginSettings
     * @return void
     */
    public function setPluginSettings($pluginSettings) {
        $this->pluginSettings = $pluginSettings;
    }
}