<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

require_once dirname(dirname(__FILE__)) . '/libraries/MontonioShippingSDK.php';

class Montonio_Shipping {

    protected $admin_notices = array();
    
    /**
     * @return void
     */
    public function __construct() {
        $this->register_hooks();
    }

    /**
     * @return self
     */
    public static function create() {
        return new self();
    }

    /**
     * @return void
     */
    protected function register_hooks() {

        // Get shipping rates to display in checkout
        add_filter( 'woocommerce_package_rates', array( $this, 'get_available_shipping_methods' ), 10, 2 );

        // Add custom order status(es); needs to be registered using register_post_status()
        add_filter( 'wc_order_statuses', array( $this, 'add_custom_order_statuses' ) );

        // Modify shipping text in "order received" page
        add_filter( 'woocommerce_order_shipping_to_display', array( $this, 'modify_order_shipping_text' ), 10, 2 );

        // Add action "Print shipping labels" to order bulk actions list
        add_filter( 'bulk_actions-edit-shop_order', array( $this, 'add_print_labels_action' ) );

        // Add action "Print shipping labels" to order action in detail view page
        add_filter( 'woocommerce_order_actions', array( $this, 'add_print_labels_action_single' ) );

        // Replace email placeholder(s) with relevant data
        add_filter( 'woocommerce_email_format_string', array( $this, 'replace_email_placeholders' ), 10, 2 );

        // Hide some metadata fields from order view
        add_filter( 'woocommerce_hidden_order_itemmeta', array( $this, 'hide_shipping_method_meta_data' ) );

        // Add logos to shipping methods
        add_filter( 'woocommerce_cart_shipping_method_full_label', array( $this, 'add_shipping_method_logo' ), 10, 2 );


        // Register status "Label printed"
        add_action( 'init', array( $this, 'register_label_printed_status' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_montonio_admin_styles' ) );
        add_action( 'login_enqueue_scripts', array( $this, 'enqueue_montonio_admin_styles' ) );

        // Add custom html to shipping section in order review
        add_action( 'woocommerce_review_order_after_shipping', array( $this, 'add_pickup_points_to_checkout' ) ); 
        
        // Update order data on checkout
        add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'update_checkout_pickup_point' ) );

        // Validate fields on checkout
        add_action( 'woocommerce_after_checkout_validation', array( $this, 'validate_pickup_point' ) );

        // Create shipment in shipping api when payment completed (either from checkout or manually)
        add_action( 'woocommerce_order_status_processing', array( $this, 'create_shipment_when_payment_complete' ), 10, 2 );

        // Add button to "Edit order" page to create shipment
        add_action( 'woocommerce_admin_order_data_after_shipping_address', array( $this, 'add_create_shipment_button' ) );

        // Perform various actions when options are saved in Montonio Shipping
        add_action( 'woocommerce_update_options_montonio_shipping', array( $this, 'process_shipping_options' ) );

        // Modify orders view columns
        add_action( 'manage_shop_order_posts_custom_column', array( $this, 'modify_order_columns' ), 10, 2 );

        // Display any shipping related admin notices
        add_action( 'admin_notices', array( $this, 'display_shipping_notices' ) );

        // Add shipping options style
        add_action( 'woocommerce_cart_totals_before_shipping', array( $this, 'enqueue_shipping_options_style' ) );
        add_action( 'woocommerce_review_order_before_shipping', array( $this, 'enqueue_shipping_options_style' ) );

        // Admin notices
        add_action( 'admin_notices', array( $this, 'display_admin_notices' ), 999 );

        // Create shipment in shipping api via js request
        add_action( 'woocommerce_api_montonio_shipping_create_shipment', array( $this, 'create_shipment_from_js_request' ) );

        // Called from Montonio Shipping API
        add_action( 'woocommerce_api_montonio_shipping_api_webhook', array( $this, 'handle_shipping_api_webhook' ) );

        // Check orders' shipping methods for label creation
        add_action( 'woocommerce_api_montonio_shipping_create_labels', array( $this, 'create_labels' ) );

        // Custom hook to sync pickup points through MontonioShippingSDK
        add_action( 'montonio_shipping_sync_pickup_points', array( $this, 'sync_pickup_points' ) );
    }

    /**
     * Set up MontonioShippingSDK if necessary options exist
     * @return MontonioShippingSDK|null
     */
    protected function init_shipping_SDK() {
        $shipping_SDK = null;
        $accessKey   = get_option( 'montonio_shipping_accessKey' );
        $secretKey   = get_option( 'montonio_shipping_secretKey' );
        $environment = get_option( 'montonio_shipping_environment', 'production' );

        if ( $accessKey && $secretKey ) {
            $shipping_SDK = new MontonioShippingSDK($accessKey, $secretKey, $environment);
        }

        return $shipping_SDK;
    }

    /**
     * @param $rates
     * @param $package
     * @return mixed
     */
    public function get_available_shipping_methods( $rates, $package ) {
        if ( get_option('montonio_shipping_enabled', 'no') !== 'yes' ) {
            return $rates;
        }

        // Sync pickup points if needed
        $lastSyncedAt = get_option( 'montonio_pickup_points_synced_at' );
        $timeDiff = null;
        if ($lastSyncedAt) {
            $currentTime = time();
            $timeDiff    = $currentTime - $lastSyncedAt;
        }

        if ( ! $lastSyncedAt || !$timeDiff || $timeDiff > 24 * 60 * 60 ) { // If difference greater than 24 hours
            do_action('montonio_shipping_sync_pickup_points');
        }

        $shippingCountry = WC()->customer->get_shipping_country();
        // Go through shipping rates and unset if not available, add respective pickup points if needed
        foreach ( $rates as $key => $rate ) {
            if ( strpos($rate->method_id, 'montonio_') === false ) {
                continue;
            }

            if ( $rate->method_id === 'montonio_shipping' ) {
                unset( $rates[$key] );
                continue;
            }

            $metaData = $rate->get_meta_data();
            $className = $metaData['method_class_name'];
            $instanceId = $metaData['instance_id'];
            $classInstance = new $className($instanceId);
            if ( ! $classInstance->isShippingMethodAvailable( $package ) ) {
                unset( $rates[$key] );
                continue;
            }

            $providerName = $metaData['provider_name'];
            $type = $metaData['type'];
            $needsPickupPoints = in_array($type, ['parcel_machine', 'post_office']);

            if ( ! $needsPickupPoints ) {
                continue;
            }

            $pickupPointsOptionName = 'montonio_' . $shippingCountry . '_' . $providerName . '_' . $type;
            $availablePickupPoints = json_decode( get_option($pickupPointsOptionName, '' ) );

            if ( ! $availablePickupPoints ) {
                unset($rates[$key]);
            }

            $rate->montonio_pickup_points_option_name = $pickupPointsOptionName;
        }

        return $rates;
    }

    /**
     * @return void
     */
    public function add_pickup_points_to_checkout() {
        $enqueueMode = get_option( 'montonio_shipping_enqueue_mode', 'enqueue' );
        if ( $enqueueMode === 'echo' ) {
            echo '<link rel="stylesheet" href="' . WC_MONTONIO_PLUGIN_URL . '/shipping/assets/css/pickup-points.css?ver=' . WC_MONTONIO_PLUGIN_VERSION . '">';
        } else {
            wp_enqueue_style( 'montonio-pickup-points', WC_MONTONIO_PLUGIN_URL . '/shipping/assets/css/pickup-points.css', array(), WC_MONTONIO_PLUGIN_VERSION );
        }

        if ( ! wp_script_is('selectWoo', 'registered') && get_option('montonio_shipping_register_selectWoo', 'no') === 'yes' ) {
            wp_register_script( 'selectWoo', WC()->plugin_url() . '/assets/js/selectWoo/selectWoo.full.min.js', array( 'jquery' ) );
        }

        if ( $enqueueMode === 'echo' ) {
            echo '<script type="text/javascript" src="'. WC_MONTONIO_PLUGIN_URL . '/shipping/assets/js/montonio-pickup-point-select.js?ver=' . WC_MONTONIO_PLUGIN_VERSION . '"></script>';
        } else {
            wp_enqueue_script( 'montonio-pickup-point-select', WC_MONTONIO_PLUGIN_URL . '/shipping/assets/js/montonio-pickup-point-select.js', array('selectWoo'), WC_MONTONIO_PLUGIN_VERSION );
        }

        $chosenShippingMethods = WC()->session->get( 'chosen_shipping_methods' );
        if ( ! $chosenShippingMethods || !is_array( $chosenShippingMethods ) ) { 
            return;
        }
        $chosenShippingMethodId = $chosenShippingMethods[0];
        if ( ! $chosenShippingMethodId || strpos( $chosenShippingMethodId, 'montonio_' ) === false ) {
            return;
        }

        $pickupPoints = $this->get_pickup_points_for_chosen_shipping_method($chosenShippingMethodId);
        if (empty($pickupPoints)) {
            return;
        }
        $pickupPoints = $this->sort_pickup_points($pickupPoints);
        $placeholder = __('Select pickup point', 'montonio-for-woocommerce');

        wc_get_template(
            'montonio/pickup-point-select.php',
            [
                'chosenShippingMethodId' => $chosenShippingMethodId,
                'placeholder' => $placeholder,
                'pickupPoints' => $pickupPoints
            ],
            '',
            WC_MONTONIO_PLUGIN_PATH . '/templates/'
        );
    }

    /**
     * @param $orderId
     * @return void
     */
    public function update_checkout_pickup_point($orderId)
    {
        $chosenPickupPointId = isset($_POST['montonio_pickup_point']) ? $_POST['montonio_pickup_point'] : null;

        $chosenShippingMethods = WC()->session->get('chosen_shipping_methods');
        if (!$chosenShippingMethods || !is_array($chosenShippingMethods)) { 
            return;
        }
        $chosenShippingMethodId = $chosenShippingMethods[0];

        if (
            !$orderId ||
            !$chosenPickupPointId ||
            !$chosenShippingMethodId ||
            strpos($chosenShippingMethodId, 'montonio_') === false
        ) {
            return;
        }

        $pickupPoints = $this->get_pickup_points_for_chosen_shipping_method($chosenShippingMethodId);
        if (empty($pickupPoints)) {
            return;
        }

        // Search for pickup point with matching id
        $chosenPickupPoint = null;
        foreach ($pickupPoints as $pickupPoint) {
            if ($pickupPoint->uuid === $chosenPickupPointId) {
                $chosenPickupPoint = $pickupPoint;
                break;
            }
        }

        if (!$chosenPickupPoint) {
            return;
        }

        if ( get_option( 'woocommerce_feature_custom_order_tables_enabled' ) == 'yes' ) {
            $order = wc_get_order( $orderId );
            $order->update_meta_data( '_montonio_pickup_point_name', $chosenPickupPoint->name);
            $order->update_meta_data( '_montonio_pickup_point_uuid', $chosenPickupPointId);
            $order->update_meta_data( '_shipping_address_1', $chosenPickupPoint->name);
            $order->update_meta_data( '_shipping_address_2', '');
            $order->update_meta_data( '_shipping_city', $chosenPickupPoint->locality);
            $order->add_meta_data( '_shipping_postcode', '');
            $order->save();
        } else {
            update_post_meta($orderId, '_montonio_pickup_point_name', $chosenPickupPoint->name);
            update_post_meta($orderId, '_montonio_pickup_point_uuid', $chosenPickupPointId);
            update_post_meta($orderId, '_shipping_address_1', $chosenPickupPoint->name);
            update_post_meta($orderId, '_shipping_address_2', '');
            update_post_meta($orderId, '_shipping_city', $chosenPickupPoint->locality);
            update_post_meta($orderId, '_shipping_postcode', '');
        }
    }

    /**
     * @param $fields
     * @return void
     */
    public function validate_pickup_point( $fields ) {
        $chosenShippingMethods = WC()->session->get('chosen_shipping_methods');
        if ( ! $chosenShippingMethods || ! is_array( $chosenShippingMethods ) ) { 
            return;
        }

        $chosenShippingMethodId = $chosenShippingMethods[0];
        if ( ! $chosenShippingMethodId || strpos( $chosenShippingMethodId, 'montonio_' ) === false ) {
            return;
        }

        if ( isset( $_POST['montonio_pickup_point'] ) && empty( $_POST['montonio_pickup_point'] ) ) {
            wc_add_notice( __( 'Please select a pickup point.', 'montonio-for-woocommerce' ), 'error' );
        }
    }

    /**
     * @return void
     */
    public function sync_pickup_points() {        
        $pickupPoints = array();
        $shipping_SDK  = $this->init_shipping_SDK();

        if ( ! $shipping_SDK ) {
            $this->add_admin_notice( __( 'Please add Montonio API keys!', 'montonio-for-woocommerce' ), 'error' );
            return;
        }

        try {    
            $pickupPoints = $shipping_SDK->get_pickup_points();

            // Divide pickup points into multiple rows in database
            update_option( 'montonio_pickup_points_synced_at', time(), 'yes' );
            $hasAnySyncFailed = false;
            foreach ( $pickupPoints as $country => $pickupPointsPerCountry ) {
                foreach ( $pickupPointsPerCountry->providers as $provider => $pickupPointsPerProvider ) {
                    foreach ( $pickupPointsPerProvider as $type => $pickupPointsPerType ) {
                        $optionName = 'montonio_' . $country . '_' . $provider . '_' . $type;
                        $pickupPointJson = json_encode($pickupPointsPerType );
                        if ( get_option($optionName) === $pickupPointJson ) {
                            continue;
                        }

                        $isUpdated = update_option( $optionName, $pickupPointJson, 'yes' );
                        $hasAnySyncFailed = !$isUpdated ?: $hasAnySyncFailed;
                    }
                }
            }

            if ( $hasAnySyncFailed ) {
                throw new Exception( __( 'Montonio Shipping was unable to sync pickup points in the background. Please try again later!', 'montonio-for-woocommerce' ) );
            }

            $this->add_admin_notice( __( 'Montonio Shipping: Pickup point sync successful!', 'montonio-for-woocommerce' ), 'success' );

            update_option( 'montonio_shipping_is_shipping_set_up', true );

        } catch ( Exception $e ) {
            if ( ! empty( $e->getMessage() ) ) {
                $this->add_admin_notice( __( 'Montonio API response: ', 'montonio-for-woocommerce' ) . $e->getMessage(), 'error' );
            }
        }
    }

    /**
     * @param $orderId
     * @param $order
     */
    public function create_shipment_when_payment_complete( $orderId, $order ) {
        if ( ! $order ) {
            return;
        }

        // Check if order has Montonio shipping method and no tracking code has already been generated
        $shippingMethod = $this->get_montonio_shipping_method( $order );
        if ( ! $shippingMethod || $shippingMethod->get_meta( 'tracking_codes' ) ) {
            return;
        }

        $this->create_shipment( $order, $shippingMethod, true );
    }

    /**
     * used to create a shipment in api manually via a button in wc order page
     */
    public function create_shipment_from_js_request() {
        $orderId = $_POST['orderId'];
        $order = wc_get_order( $orderId );
        if ( ! $order ) {
            wp_send_json_error( "SHIPMENT_CREATION_FAILED", 409 );
            return;
        }

        $shippingMethod = $this->get_montonio_shipping_method( $order );
        $response       = $this->create_shipment( $order, $shippingMethod );

        if ( $response ) {
            wp_send_json_success("SHIPMENT_CREATED", 200);
        } else {
            wp_send_json_error("SHIPMENT_CREATION_FAILED", 409);
        }
    }

    /**
     * @param $order
     */
    public function add_create_shipment_button( $order ) {
        if ( get_option('montonio_shipping_enabled', 'no' ) !== 'yes' ) {
            return;
        }

        wp_enqueue_script( 'montonio-create-shipment-button', WC_MONTONIO_PLUGIN_URL . '/shipping/assets/js/montonio-create-shipment-button.js', array(), WC_MONTONIO_PLUGIN_VERSION );

        $inlineScriptData = '
            var orderId = ' . $order->get_id() . ';
            var callbackUrl = "' . add_query_arg( 'wc-api', 'montonio_shipping_create_shipment', trailingslashit( get_home_url() ) ) . '";
        ';
        wp_add_inline_script('montonio-create-shipment-button', $inlineScriptData, 'before');

        echo '
            <p style="margin-top:20px">
                <button
                    id="montonio-shipping-create-shipment"
                    type="button" class="button button-primary"
                    style="float:left">
                    Create shipment in Montonio
                </button>
                <span id="montonio-shipping-create-shipment-spinner" class="spinner" style="float:left"></span>
            </p>
        ';
    }

    /**
     * Perform some actions when Montonio Shipping settings are saved
     */
    public function process_shipping_options() {
        $this->register_tracking_webhook();
        $this->sync_pickup_points();
    }

    /**
     * Send webhook address to shipping api for updating tracking numbers on orders
     */
    public function register_tracking_webhook() {
        $shipping_SDK = $this->init_shipping_SDK();

        if ( ! $shipping_SDK ) {
            return;
        }

        try {
            $webhook_url = apply_filters( 'wc_montonio_shipping_webhook_url', add_query_arg( 'wc-api', 'montonio_shipping_api_webhook', trailingslashit( get_home_url() ) ) );
            $response = $shipping_SDK->register_tracking_webhook( $webhook_url );
        } catch ( Exception $e ) {
            return;
        }
    }

    public function handle_shipping_api_webhook() {
        if (!isset($_REQUEST['webhook_token'])) {
            http_response_code(400);
            return;
        }
        $token = sanitize_text_field($_REQUEST['webhook_token']);

        try {
            $response = MontonioShippingSDK::decode_webhook_token($token, get_option('montonio_shipping_secretKey'));
        } catch (Exception $exception) {
            http_response_code(401);
            return;
        }

        $webhookMessage = sanitize_text_field($response->message);

        switch($webhookMessage) {
            case 'shipment_created':
                $this->handle_shipment_created_webhook($response);
                break;
            case 'label_created':
                $this->handle_label_created_webhook($response);
                break;
            default:
                break;
        }
    }

    /**
     * Register status "Label printed"
     */
    public function register_label_printed_status() {
        register_post_status('wc-mon-label-printed', array(
            'label'                     => _x('Label printed', 'Order status', 'montonio-for-woocommerce'),
            'public'                    => true,
            'exclude_from_search'       => false,
            'show_in_admin_all_list'    => true,
            'show_in_admin_status_list' => true,
            'label_count'				=> _n_noop(
                'Label printed (%s)',
                'Label printed (%s)',
                'montonio-for-woocommerce'
            )
        ));
    }

    public function enqueue_montonio_admin_styles() {
        wp_enqueue_style( 'montonio-admin-styles', WC_MONTONIO_PLUGIN_URL . '/shipping/assets/css/montonio-admin.css', array(), WC_MONTONIO_PLUGIN_VERSION );
    }

    /**
     * @param $orderStatuses
     * @return mixed
     */
    public function add_custom_order_statuses($orderStatuses) {
        $orderStatuses['wc-mon-label-printed'] = _x('Label printed', 'Order status', 'montonio-for-woocommerce');
        return $orderStatuses;
    }

    /**
     * @param $text
     * @param $order
     * @return string
     */
    public function modify_order_shipping_text($text, $order) {
        $shippingMethod = $this->get_montonio_shipping_method($order);
        if (!$shippingMethod) {
            return $text;
        }

        $shippingMethodName = $order->get_shipping_method();
        $pickupPointName = $order->get_meta('_montonio_pickup_point_name');
        if (!$pickupPointName) {
            return $text;
        }

        $stringToReplace = $shippingMethodName . ' (' . $pickupPointName . ')';
        return str_replace($shippingMethodName, $stringToReplace, $text);
    }

    public function add_print_labels_action($actions) {
        $actions['montonio_print_labels'] = __('Print shipping labels', 'montonio-for-woocommerce');

        wp_enqueue_script( 'montonio-print-labels', WC_MONTONIO_PLUGIN_URL . '/shipping/assets/js/montonio-print-labels.js', array(), WC_MONTONIO_PLUGIN_VERSION );

        $createLabelsUrl = add_query_arg( 'wc-api', 'montonio_shipping_create_labels', trailingslashit( get_home_url() ) );

        $inlineScriptData = '
            var createLabelsUrl = "' . $createLabelsUrl . '";
        ';
        wp_add_inline_script('montonio-print-labels', $inlineScriptData, 'before');

        return $actions;
    }

    public function add_print_labels_action_single($actions, $order = null) {
        wp_enqueue_script( 'montonio-print-labels', WC_MONTONIO_PLUGIN_URL . '/shipping/assets/js/montonio-print-labels.js', array(), WC_MONTONIO_PLUGIN_VERSION );

        $createLabelsUrl = add_query_arg( 'wc-api', 'montonio_shipping_create_labels', trailingslashit( get_home_url() ) );
        $inlineScriptData = '
            var createLabelsUrl = "' . $createLabelsUrl . '";
        ';
        wp_add_inline_script('montonio-print-labels', $inlineScriptData, 'before');

        $actions['montonio_print_labels'] = __('Print shipping labels', 'montonio-for-woocommerce');
        return $actions;
    }

    public function create_labels() {
        $shipping_SDK = $this->init_shipping_SDK();

        if ( ! $shipping_SDK ) {
            wp_send_json_error( "LABEL_CREATION_FAILED", 409 );
            return;
        }

        if ( ! isset( $_POST['orderIds'] ) ) {
            wp_send_json_error( "NO_ORDER_IDS_PASSED", 409 );
            return;
        }

        $orderIds = $_POST['orderIds'];
        $storePrefix = get_option( 'montonio_shipping_order_prefix' );
        $data = [
            'merchant_references' => array_map(function($orderId) use ($storePrefix) {
                return MontonioOrderPrefixer::addPrefix($storePrefix, (string) $orderId);
            }, $orderIds)
        ];

        try {
            $response = $shipping_SDK->create_labels( $data );
            wp_send_json_success( $response );
        } catch ( Exception $e ) {
            wp_send_json_error( "LABEL_CREATION_FAILED", 409 );
        }
    }

    public function modify_order_columns($column, $orderId) {
        $order = wc_get_order($orderId);
        if (!$order) {
            return;
        }

        if ($column === 'shipping_address') {
            $this->add_tracking_codes_to_custom_column($order);
        }
    }

    public function replace_email_placeholders($string, $email) {
        $placeholder = '{montonio_tracking_info}';
        $order       = $email->object;

        if (!$order) {
            return str_replace($placeholder, '', $string);
        }

        $fullText = $this->get_montonio_tracking_info($order);
        return str_replace($placeholder, $fullText, $string);
    }

    public function hide_shipping_method_meta_data($fieldsToHide) {
        $montonioFieldsToHide = [
            'shipping_method_identifier',
            'provider_name',
            'type',
            'method_class_name',
            'tracking_codes',
            'instance_id'
        ];

        return array_merge($fieldsToHide, $montonioFieldsToHide);
    }

    public function display_shipping_notices() {
        global $current_tab;
        $isShippingSetUp = get_option('montonio_shipping_is_shipping_set_up', false);

        if ( $current_tab !== 'montonio_shipping' || $isShippingSetUp ) {
            return;
        }
        
        $this->add_admin_notice( __('Before integrating deliveries in WooCommerce, make sure the providers are activated in Montonio\'s Partner System. If you cannot see "Shipping" under your account in the Partner System, please contact support@montonio.com', 'montonio-for-woocommerce'), 'info' );
    }

    /**
     * Return appropriate multiplier to convert to cm
     * @return float|int
     */
    public static function get_dimensions_multiplier() {
        $dimensionUnit = get_option('woocommerce_dimension_unit');

        switch ($dimensionUnit) {
            case 'cm':
                return 1;
            case 'm':
                return 100;
            case 'mm':
                return 0.1;
            case 'in':
                return 2.54;
            case 'yd':
                return 91.44;
            default:
                // Unknown unit, treat as missing measurements
                return 0;
        }
    }

    /**
     * Return appropriate multiplier to convert to kg
     * @return float|int
     */
    public static function get_weight_multiplier() {
        $weightUnit = get_option('woocommerce_weight_unit');

        switch ($weightUnit) {
            case 'kg':
                return 1;
            case 'g':
                return 0.001;
            case 'lbs':
                return 0.45;
            case 'oz':
                return 0.028;
            default:
                // Unknown unit, treat as missing measurements
                return 0;
        }
    }

    public function add_shipping_method_logo($label, $rate) {
        if (
            strpos($rate->method_id, 'montonio_') === false ||
            get_option('montonio_shipping_show_provider_logos', 'no') === 'no'
        ) {
            return $label;
        }

        $metaData = $rate->get_meta_data();
        $className = $metaData['method_class_name'];
        $instanceId = $metaData['instance_id'];
        $classInstance = new $className($instanceId);

        if ($classInstance->logo) {
            return $label . '
                <br />
                <img
                  class = "montonio-shipping-provider-logo"
                  id = "' . $rate->id . '_logo"
                  src = "' . $classInstance->logo . '">
            ';
        }

        return $label;
    }

    public function enqueue_shipping_options_style() {
        $enqueueMode = get_option('montonio_shipping_enqueue_mode', 'enqueue');
        if ($enqueueMode === 'echo') {
            echo '<link rel="stylesheet" href="' . WC_MONTONIO_PLUGIN_URL . '/shipping/assets/css/shipping-options.css?ver=' . WC_MONTONIO_PLUGIN_VERSION . '">';
        } else {
            wp_enqueue_style( 'montonio-shipping-options', WC_MONTONIO_PLUGIN_URL . '/shipping/assets/css/shipping-options.css', array(),  WC_MONTONIO_PLUGIN_VERSION );
        }

        $extraCss = get_option('montonio_shipping_css', null);
        if ($extraCss) {
            wp_add_inline_style('montonio-shipping-options', $extraCss);
        }
    }

    protected function get_montonio_tracking_info($order) {
        $shippingMethod = $this->get_montonio_shipping_method($order);
        $trackingCodes = $shippingMethod ? $shippingMethod->get_meta('tracking_codes') : null;
        $trackingCodesText = __(
            get_option('montonio_email_tracking_code_text', __('Track your shipment:', 'montonio-for-woocommerce')),
            'montonio-for-woocommerce'
        );

        return $trackingCodes ? $trackingCodesText . ' ' . $trackingCodes : '';
    }

    protected function handle_shipment_created_webhook($response) {
        $orderId = MontonioOrderPrefixer::removePrefix(sanitize_text_field($response->merchant_reference));
        $order = wc_get_order($orderId);
        if (!$order) {
            http_response_code(409);
            return;
        }

        $trackingCodesText = $this->process_tracking_codes($response->tracking_numbers);
        $orderNote = 'Montonio Shipping: shipment created.';
        if ($trackingCodesText) {
            $orderNote .= '<br>Tracking codes: ' . $trackingCodesText;
        }
        $order->add_order_note($orderNote);

        $this->update_tracking_codes_metadata($order, $response->tracking_numbers);
        http_response_code(200);
    }

    protected function handle_label_created_webhook( $response ) {
        $orderId = MontonioOrderPrefixer::removePrefix( sanitize_text_field( $response->merchant_reference ) );
        $order = wc_get_order( $orderId );
        if ( ! $order ) {
            http_response_code( 409 );
            return;
        }

        $trackingCodesText = $this->process_tracking_codes( $response->tracking_numbers );
        $orderNote = 'Montonio Shipping: label printed.';
        if ( $trackingCodesText ) {
            $orderNote .= '<br>Tracking codes: ' . $trackingCodesText;
        }
        $order->add_order_note( $orderNote );

        $newStatus = get_option(' montonio_shipping_orderStatusWhenLabelPrinted', 'wc-mon-label-printed' );
        if ( $order->get_status() === 'processing' && $newStatus !== 'no-change' ) {
            $order->update_status( $newStatus );
        }

        $this->update_tracking_codes_metadata( $order, $response->tracking_numbers );
        http_response_code( 200 );
    }

    protected function update_tracking_codes_metadata( $order, $trackingCodes ) {
        $trackingCodesText = $this->process_tracking_codes( $trackingCodes );
        $shippingMethod = $this->get_montonio_shipping_method( $order );
        if ( $shippingMethod && $trackingCodesText ) {
            $shippingMethod->update_meta_data( 'tracking_codes', $trackingCodesText );
            $shippingMethod->save_meta_data();

            $rawTrackingCodes = [];
            foreach ( $trackingCodes as $trackingCode ) {
                $rawTrackingCodes[] = $trackingCode->code;
            }
            $trackingInfo = $this->get_montonio_tracking_info( $order );
            $order->update_meta_data(
                '_montonio_tracking_codes',
                implode( ', ', $rawTrackingCodes )
            );
            $order->update_meta_data( '_montonio_tracking_info', $trackingInfo );
            $order->save_meta_data();
        }
    }

    protected function process_tracking_codes( $trackingCodes ) {
        $trackingCodesText = '';
        foreach ( $trackingCodes as $trackingCode ) {
            $trackingCodesText .= '
            <a target="_blank" href=' . sanitize_text_field( $trackingCode->link ) . '>'
                . sanitize_text_field( $trackingCode->code ) .
                '</a><br>';
        }

        return $trackingCodesText;
    }

    protected function add_tracking_codes_to_custom_column( $order ) {
        $shippingMethod = $this->get_montonio_shipping_method( $order );

        if (!$shippingMethod) {
            return;
        }

        if ( $shippingMethod->get_meta('tracking_codes' ) ) {
            echo __( 'Tracking code(s)', 'montonio-for-woocommerce' ) . ':<br />' .
                $shippingMethod->get_meta('tracking_codes');
            return;
        }

        $orderDatePaid = $order->get_date_paid();
        $now = time();
        if ( ! $orderDatePaid ) {
            return;
        }

        // If enough time has passed from payment and no tracking code has been received, show error message
        if ( $now - $orderDatePaid->getTimestamp() > 5 * 60 ) {
            echo '<span style="color:sandybrown">'
                . __('Unexpected error - Check status from Montonio Partner System', 'montonio-for-woocommerce')
                . '</span><br />';
        } else {
            echo '<span style="color:orange">'
                . __('Waiting for tracking codes from Montonio', 'montonio-for-woocommerce')
                . '</span><br />';
        }
    }

    protected function sort_pickup_points( $pickupPoints ) {
        // Sort pickup points by name
        usort($pickupPoints, function ( $pickupPoint1, $pickupPoint2 ) {
            return strcmp( $pickupPoint1->name, $pickupPoint2->name );
        });

        $pickupPointCount = count( $pickupPoints );
        // Count the number of pickup points in each place
        $placeNameCounts = $this->get_place_name_counts( $pickupPoints );

        $eelistusOmnivas = array();
        $pickupPointsByLargePlaces = array();
        $pickupPointsByOtherPlaces = array();
        $unspecifiedPickupPoints = array();

        foreach ($pickupPoints as $pickupPoint) {
            $hasLocality = isset( $pickupPoint->locality ) && $pickupPoint->locality !== '';
            $hasRegion = isset( $pickupPoint->region) && $pickupPoint->region !== '';
            $placeName = $hasLocality ? $pickupPoint->locality : ( $hasRegion ? $pickupPoint->region : '--' );
            $placeNameCount = isset( $placeNameCounts[$placeName] ) ? $placeNameCounts[$placeName] : 0;

            // No locality or region
            if ( $placeName === '--' ) {
                $arrayToUse = &$unspecifiedPickupPoints;
            } elseif ( $placeName === '1. eelistus Omnivas' ) {
                $arrayToUse = &$eelistusOmnivas;
            // Place has relatively many pickup points
            } elseif ( $placeNameCount >= $pickupPointCount * 0.03 ) {
                $arrayToUse = &$pickupPointsByLargePlaces;
            // Place has relatively few pickup points
            } else {
                $arrayToUse = &$pickupPointsByOtherPlaces;
            }

            if ( ! isset( $arrayToUse[$placeName] ) ) {
                $arrayToUse[$placeName] = array();
            }

            array_push($arrayToUse[$placeName], $pickupPoint);
        }

        // Sort large places by number of pickup points
        uasort($pickupPointsByLargePlaces, function($locality1, $locality2) {
            return count($locality1) >= count($locality2) ? -1 : 1;
        });

        // Sort other places alphabetically
        ksort($pickupPointsByOtherPlaces);

        return array_merge($eelistusOmnivas, $pickupPointsByLargePlaces, $pickupPointsByOtherPlaces, $unspecifiedPickupPoints);
    }

    protected function get_place_name_counts( $pickupPoints ) {
        $placeNameCounts = array();
        foreach ( $pickupPoints as $pickupPoint ) {
            if ( isset( $pickupPoint->locality ) && $pickupPoint->locality !== '' ) {
                $placeName = $pickupPoint->locality;
            } else if ( isset( $pickupPoint->region ) && $pickupPoint->region !== '' ) {
                $placeName = $pickupPoint->region;
            } else {
                continue;
            }

            if (!isset($placeNameCounts[$placeName])) {
                $placeNameCounts[$placeName] = 0;
            }

            $placeNameCounts[$placeName]++;
        }

        return $placeNameCounts;
    }

    /**
     * @param $order
     * @param null $shippingMethod
     * @param bool $async
     * @return array
     */
    protected function create_shipment( $order, $shippingMethod = null, $async = false ) {
        $shipping_SDK = $this->init_shipping_SDK();

        if ( ! $shipping_SDK ) {
            return;
        }

        $shippingMethodIdentifier = $shippingMethod && $shippingMethod->get_meta_data() ?
            $shippingMethod->get_meta( 'shipping_method_identifier' ) :
            null;

        $data = $this->get_create_shipment_data( $order, $shippingMethodIdentifier );
        
        try {
            return $shipping_SDK->post_shipment( $data, $async );
        } catch ( Exception $e ) {
            return;
        }
    }

    /**
     * @param $chosenShippingMethodId
     * @return array
     */
    protected function get_pickup_points_for_chosen_shipping_method( $chosenShippingMethodId ) {
        $packages = WC()->shipping->get_packages();

        if ( ! $packages || ! isset( $packages[0]['rates'] ) ) {
            return [];
        }

        $chosenShippingMethod = null;
        foreach ( $packages[0]['rates'] as $shippingMethod ) {
            if ( $shippingMethod->id === $chosenShippingMethodId ) {
                $chosenShippingMethod = $shippingMethod;
                break;
            }
        }

        if ( ! $chosenShippingMethod ) {
            return [];
        }

        // Backwards compatibility for checkout sessions cached before version 4.2.0
        if ( isset( $chosenShippingMethod->montonio_pickup_points ) ) {
            return $chosenShippingMethod->montonio_pickup_points;
        }

        if ( isset( $chosenShippingMethod->montonio_pickup_points_option_name ) ) {
            return json_decode( get_option($chosenShippingMethod->montonio_pickup_points_option_name, '' ) );
        }

        return [];
    }

    protected function get_create_shipment_data( $order, $shippingMethodIdentifier ) {
        $storePrefix = get_option( 'montonio_shipping_order_prefix' );
        $data = array(
            'currency'                  => (string) $order->get_currency(),
            'shipping_total'            => (float) $order->get_shipping_total(),
            'total'                     => (float) $order->get_total(),
            'billing_first_name'        => (string) $order->get_billing_first_name(),
            'billing_last_name'         => (string) $order->get_billing_last_name(),
            'billing_company'           => (string) $order->get_billing_company(),
            'billing_street_address_1'  => (string) $order->get_billing_address_1(),
            'billing_street_address_2'  => (string) $order->get_billing_address_2(),
            'billing_locality'          => (string) $order->get_billing_city(),
            'billing_region'            => (string) $order->get_billing_state(),
            'billing_postal_code'       => (string) $order->get_billing_postcode(),
            'billing_country'           => (string) $order->get_billing_country(),
            'billing_email'             => (string) $order->get_billing_email(),
            'billing_phone_number'      => (string) $order->get_billing_phone(),
            'shipping_first_name'       => (string) $order->get_shipping_first_name(),
            'shipping_last_name'        => (string) $order->get_shipping_last_name(),
            'shipping_company'          => (string) $order->get_shipping_company(),
            'shipping_street_address_1' => (string) $order->get_shipping_address_1(),
            'shipping_street_address_2' => (string) $order->get_shipping_address_2(),
            'shipping_locality'         => (string) $order->get_shipping_city(),
            'shipping_region'           => (string) $order->get_shipping_state(),
            'shipping_postal_code'      => (string) $order->get_shipping_postcode(),
            'shipping_country'          => (string) $order->get_shipping_country(),
            'shipping_phone_number'     => method_exists( $order, 'get_shipping_phone' ) ?
                (string) $order->get_shipping_phone() :
                null,
            'payment_method'            => (string) $order->get_payment_method(),
            'merchant_reference'        => (string) MontonioOrderPrefixer::addPrefix( $storePrefix, $order->get_id() ),
            'sender_name'               => (string) get_option('montonio_shipping_senderName'),
            'sender_phone_number'       => (string) get_option('montonio_shipping_senderPhone'),
            'sender_street_address_1'   => (string) get_option('montonio_shipping_senderStreetAddress'),
            'sender_locality'           => (string) get_option('montonio_shipping_senderLocality'),
            'sender_postal_code'        => (string) get_option('montonio_shipping_senderPostalCode'),
            'sender_region'             => (string) get_option('montonio_shipping_senderRegion'),
            'sender_country'            => (string) get_option('montonio_shipping_senderCountry')
        );

        if ( $shippingMethodIdentifier ) {
            $data['shipping_method'] = (string) $shippingMethodIdentifier;
        }

        $pickupPointUuid = $order->get_meta( '_montonio_pickup_point_uuid' );
        if ( $pickupPointUuid ) {
            $data['pickup_point_uuid'] = (string) $pickupPointUuid;
        }

        $parcelWeight = 0;
        foreach ( $order->get_items() as $item ) {
            $product = $item->get_product();
            $parcelWeight += (float) $product->get_weight() * $item->get_quantity() * self::get_weight_multiplier();
        }

        $data['parcels'] = array(
            array(
                'weight' => $parcelWeight
            )
        );

        return $data;
    }

    protected function get_montonio_shipping_method( $order ){
        if ( ! method_exists($order, 'get_shipping_methods' ) ) {
            return null;
        }

        $shippingMethod = null;
        foreach( $order->get_shipping_methods() as $shippingItem ) {
            $methodId = $shippingItem->get_method_id();
            if ( strpos($methodId, 'montonio_' ) !== false ) {
                $shippingMethod = $shippingItem;
                break;
            }
        }

        return $shippingMethod;
    }

    // =========================================================================
    // Admin notices
    // =========================================================================

    /**
     * Display admin notices
     */
    public function add_admin_notice( $message, $class ) {
        $this->admin_notices[] = array( 'message' => $message, 'class' => $class );
	}

    public function display_admin_notices() {
		foreach ( $this->admin_notices as $notice ) {
			echo '<div class="notice notice-' . esc_attr( $notice['class'] ) . '">';
			echo '	<p>' . wp_kses_post( $notice['message'] ) . '</p>';
			echo '</div>';
		}
	}
}
