<?php
defined('ABSPATH') or exit;

require_once 'class-montonio-split-option.php';
require_once 'class-montonio-split-settings.php';

if (!class_exists('MontonioHelper')) {
    require_once dirname(dirname(__FILE__)) . '/libraries/MontonioHelper.php';
}

class Montonio_Split extends WC_Payment_Gateway
{
    const SPLIT_MAX_CART_TOTAL = 10000;


    public function __construct($setup = true)
    {
        $this->id                 = 'montonio_split';
        $this->has_fields         = false;
        $this->method_title       = __('Montonio Pay Later', 'montonio-for-woocommerce');
        $this->method_description = __('Allows slicing the order total into a few smaller payments', 'montonio-for-woocommerce');
        $this->icon = 'https://public.montonio.com/logo/financing/en-split-3.png';
        $this->init_form_fields();
        $this->init_settings();

        $this->logo_locale = WC_Montonio_Helper::get_locale(apply_filters('wpml_current_language', get_locale()));

        if ($this->logo_locale == 'et') {
            $this->icon = 'https://public.montonio.com/logo/financing/ee-split-3.png';
        } elseif ($this->logo_locale == 'lt') {
            $this->icon = 'https://public.montonio.com/logo/financing/lt-split-3.png';
        }
    
        $this->title = __($this->get_option( 'titlev2', __('Pay later, no fees', 'montonio-for-woocommerce') ), 'montonio-for-woocommerce');
        $this->description = __($this->get_option( 'descriptionv2', __('Pay later in up to three payments without any additional fees. 0% interest, 0€ contract fee.', 'montonio-for-woocommerce') ), 'montonio-for-woocommerce');
       

        if ($setup) {

            /**
             * Register the callback function for when returning from Montonio
             */
            add_action('woocommerce_api_montonio_split_return', array($this, 'check_payment_response'));

            /**
             * Register payment notification webhook handler
             */
            add_action('woocommerce_api_montonio_split_notification', array($this, 'check_payment_notification'));

            /**
             * Show Montonio Split settings in Woocommerce->settings->payments
             */
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

            /**
             * Check if payment method available
             */
            add_filter( 'woocommerce_available_payment_gateways', array( $this, 'update_available_gateways' ), 10, 3 );


            /**
             * Add CSS class to Montonio Logo at checkout
             */
            add_filter('woocommerce_gateway_icon', array($this, 'add_montonio_logo_callback'), 10, 3);
        }
    }

    /**
     * Setup how Montonio Payments looks at checkout
     *
     * @return string - the description HTML
     */
    public function get_description()
    {
        $enqueueMode = $this->get_option('montonioSplitEnqueueMode', 'enqueue');

        if ($enqueueMode == 'echo') {
            echo '<link rel="stylesheet" href="' . WC_MONTONIO_PLUGIN_URL . '/split/assets/css/payment-handle.css">';
        } else {
            wp_enqueue_style(
                'montonio-split-payment-handle-logo',
                WC_MONTONIO_PLUGIN_URL . '/split/assets/css/payment-handle.css'
            );
        }

        return $this->description;
    }

    // =========================================================================
    // Checkout style
    // =========================================================================
    public function add_montonio_logo_callback($icon, $id)
    {
        if (strpos($id, 'montonio_split') !== false) {
            return str_replace('src="', 'id="montonio-' . $id . '-checkout-logo" src="', $icon);
        }
        return $icon;
    }

    // =========================================================================
    // Plugin settings and setup
    // =========================================================================

    public function init_form_fields()
    {
        if (!class_exists('Montonio_Split_Settings')) {
            require_once 'class-montonio-split-settings.php';
        }
        $settings = Montonio_Split_Settings::create();
        $this->setPluginSettings($settings);
        $settings->compose();
        $this->form_fields = $this->getPluginSettings()->getFormFields();
    }

    public function init_settings()
    {
        parent::init_settings();

        $settingsClass = $this->getPluginSettings();
        $settingsClass->setSettings($this->settings);
    }

    // =========================================================================
    // Split in the Checkout
    // =========================================================================
    public function update_available_gateways($available_gateways) {
        if ( ! is_admin() ) {
            if ( $this->enabled !== 'yes' ) {
                return $available_gateways;
            }

            // Check if supported currency
            if (!MontonioHelper::isClientCurrencySupported(array('EUR'))) {
                unset($available_gateways['montonio_split']);
            }

            // Check if supported locales
            $lang = apply_filters('wpml_current_language', get_locale());
            $locale = MontonioHelper::getLocale($lang);

            if (in_array($locale, array('fi', 'fi_FI', 'fi_fi', 'fi-FI', 'fi-fi'))) {
                unset($available_gateways['montonio_split']);
            }

            if (WC()->cart && WC()->cart->get_total('raw')) {
                $cartTotal = (float) (WC()->cart->get_total('raw'));
                $minCartTotal = $this->get_option('minOrderTotal', 30);
                $minCartTotal = is_numeric($minCartTotal) ? (float)$minCartTotal : 30;
                if ($cartTotal < $minCartTotal || $cartTotal > self::SPLIT_MAX_CART_TOTAL) {
                    unset($available_gateways['montonio_split']);
                }
            }
        }

        return $available_gateways;
    }


    // =========================================================================
    // Payment Creation
    // =========================================================================

    /**
     * Process Split
     *
     * @param int $order_id Order ID.
     * @return array
     */
    public function process_payment($order_id)
    {
        if (!class_exists('MontonioSplitSDK')) {
            require_once dirname(dirname(__FILE__)) . '/libraries/MontonioSplitSDK.php';
        }

        if (!class_exists('MontonioOrderPrefixer')) {
            require_once dirname(dirname(__FILE__)) . '/libraries/MontonioOrderPrefixer.php';
        }

        /**
         * Get the newly created order and prepare it for submission
         */
        $order = wc_get_order($order_id);
        $order->add_order_note(__('Montonio Split: Checkout process is started', 'montonio-for-woocommerce'));

        $merchantName = $this->get_option('montonioSplitMerchantName');
        $storePrefix  = $this->get_option('montonioSplitOrderPrefix');

        /**
         * Create new Montonio Split SDK instance
         */
        $montonioSplit = new MontonioSplitSDK(
            $this->get_option('montonioSplitAccessKey'),
            $this->get_option('montonioSplitSecretKey'),
            $this->get_option('montonioSplitEnvironment')
        );

        /**
         * Prepare Payment Data for Montonio Split
         */
        $paymentData = array(
            'amount'                    => $order->get_total(),
            'currency'                  => $order->get_currency(),
            'merchant_reference'        => MontonioOrderPrefixer::addPrefix($storePrefix, $order_id),
            'merchant_name'             => $merchantName,
            'checkout_email'            => (string) $order->get_billing_email(),
            'checkout_first_name'       => (string) $order->get_billing_first_name(),
            'checkout_last_name'        => (string) $order->get_billing_last_name(),
            'checkout_phone_number'     => (string) $order->get_billing_phone(),
            'checkout_city'             => (string) $order->get_billing_city(),
            'checkout_address'          => (string) $order->get_billing_address_1(),
            'checkout_postal_code'      => (string) $order->get_billing_postcode(),
            'merchant_notification_url' => add_query_arg('wc-api', 'montonio_split_notification', home_url('/')),
            'merchant_return_url'       => add_query_arg('wc-api', 'montonio_split_return', home_url('/')),
            'preselected_loan_period'   => 3
        );

        $paymentData['checkout_products'] = $this->_prepareCheckoutProducts($order);

        /**
         * Add preselected locale if possible
         */
        $wpmlCustomerLanguage = apply_filters('wpml_current_language', null);
        if ($wpmlCustomerLanguage) {
            $paymentData['preselected_locale'] = MontonioHelper::getLocale($wpmlCustomerLanguage);
        } else {
            $paymentData['preselected_locale'] = MontonioHelper::getLocale(get_locale());
        }

        $paymentData = apply_filters('montonio_split_before_payment_data_submission', $paymentData, $order);

        $montonioSplit->setPaymentData($paymentData);

        $paymentUrl = $montonioSplit->getPaymentUrl();

        /**
         * Return response after which redirect to Montonio Split will happen
         */
        return array(
            'result'   => 'success',
            'redirect' => $paymentUrl,
        );
    }

    protected function _prepareCheckoutProducts($order)
    {

        $products = array();

        foreach ($order->get_items() as $item_id => $item_data) {
            $product       = $item_data->get_product();
            $item_quantity = $item_data->get_quantity(); // Get the item quantity
            $product_name  = $product->get_name(); // Get the product name

            $products[] = array(
                "product_name"  => $product_name,
                "product_price" => 0,
                "quantity"      => (int) $item_quantity,
            );
        }

        $shipping_cost = $order->get_shipping_total();
        if ($shipping_cost && $shipping_cost > 0) {
            $products[] = array(
                "product_name"  => 'SHIPPING',
                "product_price" => 0,
                "quantity"      => 1,
            );
        }

        // actual total of the cart
        $products[] = array(
            "product_name"  => 'Cart total',
            "product_price" => $order->get_total(),
            "quantity"      => 1
        );

        return $products;
    }

    public function check_payment_notification()
    {
        return $this->check_payment_response(true);
    }

    /**
     * Check callback from Montonio
     * and redirect user: thankyou page for success, checkout on declined/failure
     *
     * @param bool $is_notification True if webhook notification, false otherwise
     *
     * @return void
     */
    public function check_payment_response($is_notification = false)
    {
        global $woocommerce;

        /**
         * Get Payment Token from request
         */
        if (!isset($_REQUEST['payment_token'])) {
            wc_add_notice(__('Unable to finish the payment. Please try again or choose a different payment method.', 'montonio-for-woocommerce'), 'notice');
            wp_redirect(wc_get_checkout_url());
            exit;
        }

        $token = sanitize_text_field($_REQUEST['payment_token']);

        /**
         * Decode and Validate Payment Token with Montonio Split SDK
         */
        if (!class_exists('MontonioSplitSDK')) {
            require_once dirname(dirname(__FILE__)) . '/libraries/MontonioSplitSDK.php';
        }

        if (!class_exists('MontonioOrderPrefixer')) {
            require_once dirname(dirname(__FILE__)) . '/libraries/MontonioOrderPrefixer.php';
        }

        $response = MontonioSplitSDK::decodePaymentToken($token, $this->get_option('montonioSplitSecretKey'));

        $paymentStatus = sanitize_text_field($response->status);

        $orderId = MontonioOrderPrefixer::removePrefix(sanitize_text_field($response->merchant_reference));

        /**
         * Get order
         */
        $order = wc_get_order($orderId);

        /**
         * Check that Montonio was used to place the order
         */
        if (!$order || substr($order->get_payment_method(), 0, strlen('montonio_split')) !== 'montonio_split') {
            http_response_code(404);
            die();
        }

        /**
         * Return URL if payment for order failed
         */
        $returnUrl = wc_get_checkout_url();

        /**
         * Add ASPSP name to order notes if it was provided
         */
        $comment = null;
        if ($response->payment_method_name) {
            $comment = 'Montonio Payment Method: ' . $response->payment_method_name;
        }

        if (!empty($order->get_date_paid())) {
            $returnUrl = WC_Payment_Gateway::get_return_url($order);
        } else {
            if ($paymentStatus === 'finalized') {
                $order->payment_complete();
                if ($comment) {
                    $order->add_order_note($comment);
                }
                $returnUrl = WC_Payment_Gateway::get_return_url($order);
                $woocommerce->cart->empty_cart();
            } else {
                wc_add_notice(__('Unable to finish the payment. Please try again or choose a different payment method.', 'montonio-for-woocommerce'), 'notice');
            }
        }

        if ($is_notification) {
            http_response_code(200);
        } else {
            wp_redirect($returnUrl);
        }

        exit;
    }

    /**
     * @return object
     */
    public function getPluginSettings()
    {
        return $this->pluginSettings;
    }

    /**
     * @param object $pluginSettings
     */
    public function setPluginSettings($pluginSettings)
    {
        $this->pluginSettings = $pluginSettings;
    }
}
