<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * API for Montonio Payments.
 * This class contains methods for starting and validating payments.
 */
class WC_Montonio_API {
    
    /**
     * Payment Data for Montonio Payment Token generation
     * @see https://docs.montonio.com/api/payments/#generating-the-token
     *
     * @var array
     */
    public $payment_data;

    /**
     * Montonio Access Key
     *
     * @var string
     */
    public $access_key;

    /**
     * Montonio Secret Key
     *
     * @var string
     */
    public $secret_key;

    /**
     * Montonio Environment (Use sandbox for testing purposes)
     *
     * @var string 'yes'
     */
    public $sandbox_mode;

    /**
     * Root URL for the Montonio Sandbox application
     */
    const MONTONIO_SANDBOX_API_URL = 'https://sandbox-stargate.montonio.com/api';

    /**
     * Root URL for the Montonio application
     */
    const MONTONIO_API_URL = 'https://stargate.montonio.com/api';

    public function __construct( $access_key, $secret_key, $sandbox_mode ) {
        $this->access_key   = $access_key;
        $this->secret_key   = $secret_key;
        $this->sandbox_mode = $sandbox_mode;
    }

    /**
     * Get the URL string where to redirect the customer to
     *
     * @return string
     */
    public function get_payment_url() {
        $response = $this->create_order();

        return json_decode( $response )->paymentUrl;
    }

    /**
     * Generate JWT from order data
     *
     * @return string
     */
    protected function get_order_data() {

        $order = wc_get_order( $this->payment_data['merchantReference'] );
        $order->add_order_note( __( 'Montonio: Checkout process is started', 'montonio-for-woocommerce' ) );

        $payment_method_id = $this->payment_data['paymentMethodId'];

        $locale = '';
        $wpml_customer_language = apply_filters( 'wpml_current_language', null );
        if ( $wpml_customer_language ) {
            $locale = WC_Montonio_Helper::get_locale( $wpml_customer_language );
        } else {
            $locale = WC_Montonio_Helper::get_locale( get_locale() );
        }
        
        //Parse Order Data to correct data types and add additional data
        $order_data = array(
            'accessKey'                => (string) $this->access_key,
            'merchantReference'        => (string) $this->payment_data['merchantReference'],
            'merchantReferenceDisplay' => (string) apply_filters( 'wc_montonio_merchant_reference_display', $order->get_order_number() ),
            'notificationUrl'          => (string) apply_filters( 'wc_montonio_notification_url', add_query_arg( 'wc-api', $payment_method_id . '_notification', trailingslashit( get_home_url() ) ), $payment_method_id ),
            'returnUrl'                => (string) apply_filters( 'wc_montonio_return_url', add_query_arg( 'wc-api', $payment_method_id, trailingslashit( get_home_url() ) ), $payment_method_id ),
            'grandTotal'               => (float) $order->get_total(),
            'currency'                 => (string) $order->get_currency(),
            'locale'                   => (string) $locale,
            'billingAddress'           => array(
                'firstName'    => (string) $order->get_billing_first_name(),
                'lastName'     => (string) $order->get_billing_last_name(),
                'email'        => (string) $order->get_billing_email(),
                'phoneNumber'  => (string) $order->get_billing_phone(),
                'addressLine1' => (string) $order->get_billing_address_1(),
                'addressLine2' => (string) $order->get_billing_address_2(),
                'locality'     => (string) $order->get_billing_city(),
                'region'       => (string) $order->get_billing_state(),
                'postalCode'   => (string) $order->get_billing_postcode(),
                'country'      => (string) $order->get_billing_country(),
            ),
            'shippingAddress'           => array(
                'firstName'    => (string) $order->get_shipping_first_name(),
                'lastName'     => (string) $order->get_shipping_last_name(),
                'email'        => (string) $order->get_billing_email(),
                'phoneNumber'  => (string) $order->get_billing_phone(),
                'addressLine1' => (string) $order->get_shipping_address_1(),
                'addressLine2' => (string) $order->get_shipping_address_2(),
                'locality'     => (string) $order->get_shipping_city(),
                'region'       => (string) $order->get_shipping_state(),
                'postalCode'   => (string) $order->get_shipping_postcode(),
                'country'      => (string) $order->get_shipping_country(),
            ),
            'lineItems' => array(),
            'payment'   => array(
                'method'        => (string) $this->payment_data['payment']['method'],
                'methodDisplay' => (string) $this->payment_data['payment']['methodDisplay'],
                'amount'        => (float) $order->get_total(),
                'currency'      => (string) $order->get_currency(),
                'methodOptions' => $this->payment_data['payment']['methodOptions'],
            )
        );

        // Add products & shipping to Payment Data
        foreach ( $order->get_items() as $item_id => $item_data ) {
            $product = $item_data->get_product();
            $product_price = $item_data->get_total() + $item_data->get_total_tax();
            $order_data['lineItems'][] = array(
                'name'        => $product->get_name(),
                'finalPrice' => (float) $product_price,
                'quantity'    => (int) $item_data->get_quantity(),
            );
        }

        $shipping_price = $order->get_shipping_total() + $order->get_shipping_tax();
        if ( $shipping_price && $shipping_price > 0 ) {
            $order_data['lineItems'][] = array(
                'name'        => 'SHIPPING',
                'finalPrice' => (float) $shipping_price,
                'quantity'    => 1,
            );
        }

        $order_data = apply_filters('wc_montonio_before_order_data_submission', $order_data, $order);

        foreach ( $order_data as $key => $value ) {
            if ( empty( $value ) || $value == '' ) {
                unset( $order_data[$key] );
            }
        }
        
        // Add expiry to payment data for JWT validation
        $exp = time() + (10 * 60);
        $order_data['exp'] = $exp;

        return $order_data;
    }

    public function create_order() {
        $order_data = $this->get_order_data();
        $url = ( $this->sandbox_mode === 'yes' )
        ? self::MONTONIO_SANDBOX_API_URL
        : self::MONTONIO_API_URL;
        $url .= '/orders';

        $args = array(
            'headers' => array(
                'Content-Type' => 'application/json',
            ),
            'method' => 'POST',
            'body'   => json_encode( array( 'data' => MontonioFirebaseV2\JWT\JWT::encode($order_data, $this->secret_key, 'HS256' ) ) )
        );

        $args          = apply_filters('wc_montonio_create_order_request_args', $args);
        $response      = wp_remote_request($url, $args);
        $response_code = wp_remote_retrieve_response_code($response);

        if ( $response_code !== 200 && $response_code !== 201 ) {
           throw new Exception( wp_remote_retrieve_body( $response ) );
        }

        return wp_remote_retrieve_body( $response );
    }

    /**
     * Decode the Payment Token
     * This is used to validate the integrity of a callback when a payment was made via Montonio
     * @see https://payments-docs.montonio.com/#validating-the-returned-payment-token
     *
     * @param string $token - The Payment Token
     * @param string Your Secret Key for the environment
     * @return object The decoded Payment token
     */
    public static function decode_payment_token( $token, $secret_key ) {
        MontonioFirebaseV2\JWT\JWT::$leeway = 60 * 5; // 5 minutes
        return MontonioFirebaseV2\JWT\JWT::decode( $token, $secret_key, array( 'HS256' ) );
    }

    /**
     * Get the Bearer auth token for requests to Montonio
     *
     * @param string $access_key - Your Access Key
     * @param string $secret_key - Your Secret Key
     * @return string
     */
    static function get_bearer_token( $access_key, $secret_key ) {
        $payload = array(
            'accessKey' => $access_key,
            'iat'       => time(),
            'exp'       => time() + (60 * 60)
        );

        $token =  MontonioFirebaseV2\JWT\JWT::encode( $payload, $secret_key );

        return $token;
    }

    /**
     * Fetch info about banks and card processors that
     * can be shown to the customer at checkout.
     * 
     * @return string String containing the banklist
     */
    public function fetch_payment_methods() {
        $url = ( $this->sandbox_mode === 'yes' )
        ? self::MONTONIO_SANDBOX_API_URL
        : self::MONTONIO_API_URL;
        $url .= '/stores/setup';
        
        $args = array(
            'headers' => array(
                'Content-Type'  => 'application/json',
                'Authorization' => 'Bearer ' . WC_Montonio_API::get_bearer_token( $this->access_key, $this->secret_key )
            )
        );

        $args          = apply_filters( 'wc_montonio_fetch_methods_request_args', $args );
        $response      = wp_remote_request( $url, $args );
        $response_code = wp_remote_retrieve_response_code ( $response );

        if ( $response_code !== 200 ) {
            throw new Exception( wp_remote_retrieve_body( $response ) );
        }

        return wp_remote_retrieve_body( $response );
    }
}