<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WC_Montonio_Card extends WC_Payment_Gateway {

    protected $admin_notices = array();

    public function __construct() {
        $this->id                 = 'wc_montonio_card';
        $this->icon               = 'https://public.montonio.com/images/logos/visa-mc-ap-gp.png';
        $this->has_fields         = false;
        $this->method_title       = __( 'Montonio Card Payments (2023)', 'montonio-for-woocommerce' );
        $this->method_description = __( 'Allows card payments via Montonio', 'montonio-for-woocommerce' );
        $this->supports           = array(
            'products'
        );

        // Load the form fields.
        $this->init_form_fields();

        // Load the settings.
        $this->init_settings();

        // Get settings
        $this->title           = __( $this->get_option( 'title', __( 'Card Payment', 'montonio-for-woocommerce' ) ), 'montonio-for-woocommerce' );
        $this->description     = __( $this->get_option( 'description' ), 'montonio-for-woocommerce' );
        $this->enabled         = $this->get_option( 'enabled' );
        $this->sandbox_mode    = $this->get_option( 'sandbox_mode' );

        $api_settings     = get_option( 'woocommerce_wc_montonio_api_settings' );
		$this->access_key = ! empty( $api_settings['access_key'] ) ? $api_settings['access_key'] : '';
		$this->secret_key = ! empty( $api_settings['secret_key'] ) ? $api_settings['secret_key'] : '';

        if ( $this->sandbox_mode === 'yes' ) {
			$this->access_key = ! empty( $api_settings['sandbox_access_key'] ) ? $api_settings['sandbox_access_key'] : '';
			$this->secret_key = ! empty( $api_settings['sandbox_secret_key'] ) ? $api_settings['sandbox_secret_key'] : '';
		}


        // Hooks
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        add_filter( 'woocommerce_settings_api_sanitized_fields_' . $this->id, array( $this, 'validate_settings' ) );
        add_filter( 'woocommerce_available_payment_gateways', array( $this, 'update_available_gateways' ), 10, 3 );
        add_action( 'woocommerce_api_' . $this->id, array( $this, 'get_order_response' ) );
        add_action( 'woocommerce_api_' . $this->id . '_notification', array( $this, 'get_order_notification' ) );
        add_filter( 'woocommerce_gateway_icon', array( $this, 'add_icon_class' ), 10, 3 );
        add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );
        add_action( 'admin_notices', array( $this, 'display_admin_notices' ), 999 );
    }

    /**
     * Edit gateway icon.
     */
    public function add_icon_class($icon, $id) {
        if ($id == $this->id) {
            return str_replace('src="', 'class="montonio-payment-method-icon montonio-card-icon" src="', $icon);
        }
        
        return $icon;
    }

    /**
    * Plugin options, we deal with it in Step 3 too
    */
    public function init_form_fields() {
        $this->form_fields = array(
            'enabled'         => array(
                'title'       => __( 'Enable/Disable', 'montonio-for-woocommerce' ),
                'label'       => __( 'Enable Montonio Card', 'montonio-for-woocommerce' ),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no',
            ),
            'sandbox_mode'        => array(
                'title'       => 'Sandbox mode',
                'label'       => 'Enable Test Mode',
                'type'        => 'checkbox',
                'description' => __( 'Use the Sandbox environment for testing only.', 'montonio-for-woocommerce' ),
                'default'     => 'yes',
                'desc_tip'    => true,
            ),
            'title'           => array(
                'title'       => __( 'Title', 'montonio-for-woocommerce' ),
                'type'        => 'text',
                'default'     => __( 'Card Payment', 'montonio-for-woocommerce' ),
                'description' => __( 'Payment method title which the user sees during checkout.', 'montonio-for-woocommerce' ),
                'desc_tip'    => true,
            ),
            'description'      => array(
                'title'       => __( 'Description', 'montonio-for-woocommerce' ),
                'type'        => 'textarea',
                'css'         => 'width: 400px;',
                'default'     => __( 'Pay with your credit or debit card via Montonio.', 'montonio-for-woocommerce' ),
                'description' => __( 'Payment method description which the user sees during checkout.', 'montonio-for-woocommerce' ),
                'desc_tip'    => true,
            ),
        );
    }

    public function update_available_gateways( $available_gateways ) {
        if ( ! is_admin() ) {
            if ( $this->enabled !== 'yes' ) {
                return $available_gateways;
            }

            if ( ! WC_Montonio_Helper::is_client_currency_supported() ) {
                unset( $available_gateways['wc_montonio_card'] );
            }

            if (WC()->cart && WC()->cart->get_total('raw')) {
                $cart_total = (float) ( WC()->cart->get_total('raw') );
                
                if ( $cart_total < 0.5 ) {
                    unset( $available_gateways['wc_montonio_card'] );
                }
            }
        }

        return $available_gateways;
    }

    /**
     * Perform validation on settings after saving them
     */
    public function validate_settings( $settings ) {
        if ( is_array( $settings ) ) {

            if ( $settings['enabled'] === 'no' ) {
                return $settings;
            }

            $api_settings = get_option( 'woocommerce_wc_montonio_api_settings' );

            // Disable the payment gateway if API keys are not provided
            if ( $settings['sandbox_mode'] === 'yes' ) {
                if ( empty( $api_settings['sandbox_access_key'] ) || empty( $api_settings['sandbox_secret_key'] ) ) {
                    $this->add_admin_notice( sprintf( __( 'Sandbox API keys missing. Montonio Card Payments was disabled. <a href="%s">Add API keys here</a>.', 'montonio-for-woocommerce' ), admin_url( 'admin.php?page=wc-settings&tab=checkout&section=wc_montonio_api' ) ), 'error' );
                    $settings['enabled'] = 'no';

                    return $settings;
                }
            } else {
                if ( empty( $api_settings['access_key'] ) || empty( $api_settings['secret_key'] ) ) {
                    $this->add_admin_notice( sprintf( __( 'Live API keys missing. Montonio Card Payments was disabled. <a href="%s">Add API keys here</a>.', 'montonio-for-woocommerce' ), admin_url( 'admin.php?page=wc-settings&tab=checkout&section=wc_montonio_api' ) ), 'error' );
                    $settings['enabled'] = 'no';
                    
                    return $settings; 
                }
            }
        }

        return $settings;       
    }

    /*
    * We're processing the payments here
    */
    public function process_payment( $order_id ) {

        // Create new Montonio API instance
        $montonio_api = new WC_Montonio_API(
            $this->access_key,
            $this->secret_key,
            $this->sandbox_mode
        );

        // Prepare Payment Data for Montonio Payments
        $payment_data = array(
            'paymentMethodId'    => $this->id,
            'merchantReference' => $order_id,
            'payment'           => array(
                'method'        => 'cardPayments',
                'methodDisplay' => $this->get_title(),
                'methodOptions' => array(),
            ),
        );

        $montonio_api->payment_data = $payment_data;

        try {
            $payment_url = $montonio_api->get_payment_url();
            
            // Return response after which redirect to Montonio Payments will happen
            return array(
                'result'   => 'success',
                'redirect' => $payment_url,
            );
        } catch ( Exception $e ) {
            wc_add_notice( __( 'There was a problem processing this payment. Please refresh the page and try again.', 'montonio-for-woocommerce' ), 'error' );

            if ( ! empty( $e->getMessage() ) ) {
                wc_add_notice( __( 'Montonio Payments response: ', 'montonio-for-woocommerce' ) . $e->getMessage(), 'error' );
            }
        }
    }


    public function payment_scripts() {
        wp_enqueue_style( 'montonio-style' );
    }
    
    /**
     * Check webhook notfications from Montonio
     */
    public function get_order_notification() {
        new WC_Montonio_Webhooks( 
            $this->sandbox_mode,
            true 
        );
    }

    /**
     * Check callback from Montonio
     * and redirect user: thankyou page for success, checkout on declined/failure
     */
    public function get_order_response() {
        new WC_Montonio_Webhooks( 
            $this->sandbox_mode,
            false 
        );
    }

    /**
     * Edit settings page layout
     */
    public function admin_options() {
        WC_Montonio_Display_Admin_Options::display_options( 
            $this->method_title, 
            $this->generate_settings_html( array(), false ),
            $this->id
        );
    }

    /**
     * Display admin notices
     */
    public function add_admin_notice( $message, $class ) {
        $this->admin_notices[] = array( 'message' => $message, 'class' => $class );
	}

    public function display_admin_notices() {
		foreach ($this->admin_notices as $notice) {
			echo '<div id="message" class="' . esc_attr( $notice['class'] ) . '">';
			echo '	<p>' . wp_kses_post( $notice['message'] ) . '</p>';
			echo '</div>';
		}
	}
}
