<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles webhooks from Montonio API
 */
class WC_Montonio_Webhooks extends WC_Payment_Gateway {
    /**
     * Montonio Environment
     *
     * @var string 'yes'
     */
    public $sandbox_mode;

    /**
     * Bool if the response is webhook notification
     *
     * @var bool
     */
    public $is_notification;


	public function __construct( $sandbox_mode, $is_notification ) {
		$this->sandbox_mode = $sandbox_mode;
		$this->is_notification = $is_notification;

        $api_settings     = get_option( 'woocommerce_wc_montonio_api_settings' );
        $this->access_key   = ! empty( $api_settings['access_key'] ) ? $api_settings['access_key'] : '';
		$this->secret_key   = ! empty( $api_settings['secret_key'] ) ? $api_settings['secret_key'] : '';

        if ( $this->sandbox_mode === 'yes' ) {
			$this->access_key = ! empty( $api_settings['sandbox_access_key'] ) ? $api_settings['sandbox_access_key'] : '';
			$this->secret_key = ! empty( $api_settings['sandbox_secret_key'] ) ? $api_settings['sandbox_secret_key'] : '';
		}

        $this->check_order_response();
	}

    /**
     * Check callback from Montonio
     * and redirect user: thankyou page for success, checkout on declined/failure
     *
     * @param bool $is_notification True if webhook notification, false otherwise
     *
     * @return void
     */
    public function check_order_response() {

        // Get Payment Token
        if ( ! isset( $_REQUEST['order-token'] ) ) {
            wc_add_notice( __( 'Unable to finish the payment. Please try again or choose a different payment method.', 'montonio-for-woocommerce' ), 'notice' );
            wp_redirect( wc_get_checkout_url() );
            exit;
        }

        $token = sanitize_text_field( $_REQUEST['order-token'] );

        // Return URL if payment for order failed
        $return_url = wc_get_checkout_url();

        // Create new Montonio API instance
        $montonio_api = new WC_Montonio_API(
            $this->access_key,
            $this->secret_key,
            $this->sandbox_mode
        );

        try {
            $response = $montonio_api->decode_payment_token( apply_filters( 'wc_montonio_decode_payment_token', $token ), $this->secret_key ); 
        } catch ( Throwable $exception ) {
            wc_add_notice( __( 'There was a problem with processing the order.', 'montonio-for-woocommerce' ), 'error' );

            if ( $is_notification ) {
                http_response_code( 401 );
            } else {
                wp_redirect( $return_url );
            }
            exit;
        }

        $payment_status = sanitize_text_field( $response->paymentStatus );
        $order_id = sanitize_text_field( $response->merchantReference );

        $order = wc_get_order( $order_id );

        if ( ! $order ) {
            http_response_code( 404 );
            die();
        }

        if ( ! empty( $order->get_date_paid() ) ) {
            $return_url = $this->get_return_url( $order );
        } else {
            if ( $payment_status === 'PAID' ) {
                $order->payment_complete();
                $order->add_order_note( __( 'Montonio: Payment has been completed', 'montonio-for-woocommerce' ) );

                if ( $response->paymentProviderName ) {
                    $order->add_order_note( __( 'Montonio: Payment method - ', 'montonio-for-woocommerce' ) . $response->paymentProviderName );
                } elseif ( $response->paymentMethod == 'blik') {
                    $order->add_order_note( __( 'Montonio: Payment method - ', 'montonio-for-woocommerce' ) . 'BLIK' );
                } elseif ( $response->paymentMethod == 'cardPayments') {
                    $order->add_order_note( __( 'Montonio: Payment method - ', 'montonio-for-woocommerce' ) . __( 'Card payment', 'montonio-for-woocommerce' ) );
                }

                WC()->cart->empty_cart();            
                $return_url = $this->get_return_url( $order );
            } else {
                wc_add_notice( __( 'Unable to finish the payment. Please try again or choose a different payment method.', 'montonio-for-woocommerce' ), 'notice' );
            }
        }
               
        if ( $this->is_notification ) {
            http_response_code( 200 );
        } else {
            wp_redirect( $return_url );
        }

        exit;
    }
}